#!/usr/bin/perl

use LWP::UserAgent;
use HTTP::Request::Common qw(GET);
use Time::Local;

use File::Basename;

my $path=dirname($0);
require "$path/tracking.pm";

use strict;

sub dm2dd($)
{
   my ($dm) = @_;
   my ($d, $m) = split(/:/, $dm);
   if ( $d < 0 ) {
	$d -= ( $m / 60 );
   } else {
	$d += ( $m / 60 );
   }
   return $d;
}

my ($line) = `netstat -r -n | grep '^default'`;
my @content = split(/\s+/,$line);
my $iop = @content[1];

do_open_log("IOPTrack");
do_log("Starting");
do_fatal("No default route: $iop") if ( $iop !~ /[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+/ );
my $req = GET "http://${iop}/diagnostics.ssi";
my $ua = LWP::UserAgent->new;
my $res = $ua->request($req);

do_fatal("No IOP at $iop") if ( ! $res->is_success );
my @content = split(/\n/, $res->content);
chomp(@content);
my ($imei) = grep(/IMEI/, @content);
$imei =~ s/^[^0-9]+//;
$imei =~ s/[^0-9]+$//;
do_log("Serial Number(IMEI): $imei");

$req = GET "http://${iop}/text_status.ssi";
$res = $ua->request($req);
@content = split(/\n/, $res->content);
chomp(@content);
my ($gpsstat) = grep(/Status: GPS/, @content);
do_fatal($gpsstat) if ( $gpsstat !~ /fix valid/);
my ($timestamp) = grep(/Timestamp:/, @content);
$timestamp =~ s/<p>Timestamp: //;
$timestamp =~ s/<\/p>//;
my ($date, $time) = split(/ /,$timestamp);
my $day = substr($date, 0, 2);
my $mon = substr($date,2,2);
my $year = substr($date,4,2) + 2000;
my $hour = substr($time,0,2);
my $min = substr($time,2,2);
my $sec = substr($time,4,2);

my ($lat) = grep(/Latitude:/, @content);
$lat =~ s/<p>Latitude: //;
$lat =~ s/<\/p>//;
my $latd=dm2dd($lat);

my ($lon) = grep(/Longitude:/, @content);
$lon =~ s/<p>Longitude: //;
$lon =~ s/<\/p>//;
my $lond=dm2dd($lon);

#print "$day, $mon, $year, $hour, $min, $sec, $lat, $lon \n";

my $utime = timegm($sec, $min, $hour, $day, $mon-1, $year);
my $lats = "+";
$lats = "" if ( $latd < 0 );
my $lons = "+";
$lons = "" if ( $lond < 0 );

do_log("Latitude: $lat");
do_log("Longitude: $lon");
do_log("Date: $day/$mon/$year");
do_log("Time: $hour:$min:$sec");
print "$imei,$utime,${lats}${latd},${lons}${lond}\n";

do_exit("Done");
