#!/usr/bin/perl

use Getopt::Std;
use Fcntl ':flock';
use IO::File;
use IO::Handle;
use IO::Select;
use IO::Socket;
use POSIX qw(strftime);
use Data::Dumper;
use File::Path qw(make_path remove_tree);

my $ver		= "/etc/version";
my $logtime     = time();
my $LOGDIR      = "/var/log";
my $debuglog    = "$LOGDIR/ipdial.log";
my $debugging   = 1;
my $clientin    = *STDIN;
my $clientout   = *STDOUT;
my $PPP         = "/usr/sbin/ppp -quiet -background";
my $PPPSOCK     = "/var/run/ppp";
my $PPPCTL      = "/usr/sbin/pppctl $PPPSOCK";
my $PPPCONFDIR  = "/etc/ppp";
my $PPPCONFTMPL = "$PPPCONFDIR/ppp.conf.template";
my $PPPCONF     = "$PPPCONFDIR/ppp.conf";
my $ipderror    = 0;
my $pinghost	= "208.86.227.175";
my $childpid    = 0;

$clientout->autoflush(1);
binmode $clientout or do_fatal("Could not bin mode : $!");

my $phone     = "";
my $authname  = "";
my $authkey   = "";
my $speed     = "";
my $idle      = 60;
my $extrainit = "";

$SIG{CHILD} = 'IGNORE';

sub do_keepalive()
{
    close(\*STDIN);
    close(\*STDOUT);
    close(\*STDERR);
    while ( (my $ppid = getppid()) > 1 ) {
#	print "child: ppid = $ppid\n";
	sleep(5);
	system("ping -q -c 1 -t 5 $pinghost >/dev/null 2>&1");
    }
}

sub do_openport()
{
     my $stat = 0;
     if ( !is_connected() ) {
	  my $count = 15;
	  do {
              $stat = system("ping -q -c 1 -t 5 $pinghost >/dev/null 2>&1");
#	      print ".";
	  } while ( $count-- && $stat ); 
     }
     if ( $stat ) {
	 do_error("Connection failed"); 
	 return 0;
     } 
     $childpid = fork();
     if ( $childpid == 0 ) {
          do_keepalive();
	  exit(0);
     }
     ok();
#     print "parent: forked child pid = $childpid\n";
     return 1;
}

sub debug(@)
{
    if ( !$debugging )  { return } ;
    my $fmt = shift @_;
    my $str = sprintf("$fmt", @_);
    my $now_string = strftime "%a %b %e %H:%M:%S %Y", localtime;
    open(DEBUG,">>$debuglog") || die "Could not open debug log $debuglog\n";
    flock(DEBUG,LOCK_EX);
    print DEBUG "$now_string [$mypid-$logtime] $str \n";
    flock(DEBUG,LOCK_UN);
    close (DEBUG);
}   

sub do_fatal(@)
{
    my @errmsg = @_;
    print $clientout "ERROR: ";
    print $clientout @_;
    print $clientout "\r\n\r\n";
    $clientout->flush();
    debug(@errmsg);
                
    # delay some so the client can read the ERROR
    sleep(5);
    exit;
}

sub do_send($)
{
    my ($str) = @_;
    print $clientout "$str\r\n";
}

sub do_error($)
{
    my ($str) = @_;
    $ipderror = 1;
    do_send("ERROR: $str");
}

sub ok()
{
    do_send("OK");
}

sub do_php($)
{
    my ($args) = @_;
    my $cmd = "/usr/local/bin/php /usr/local/gmnbin/ipdialconf.php $args";
    my @ret = `$cmd`;
    chomp(@ret);
    my @stat = grep(/ERROR/,@ret);
    my @sum  = grep(/SUM/, @ret);
    if ( @stat > 0 )  {
 	do_send("@stat");
	return 0;
    } else {
	if ( @sum > 0 ) {
	    do_send("@sum");
	}
	ok();
    }
    return 1;
}

sub is_connected()
{
   if ( $childpid ) {
        my $stat = system("ping -q -c 2 -i 2 -t 5 $pinghost >/dev/null 2>&1");
	return ! $stat;
    }

    if ( -e "$PPPSOCK" ) {
         return 1;
    }
    return 0;

}

sub do_status()
{
    if ( is_connected() ) {
	do_send("Connected");
	return 1;
    } else {
	do_send("Disconnected");
	return 0;
    }
}

sub do_dialup($)
{
    my ($arg) = @_;
    $arg = "genericwinit" if ( $arg eq "generic" && $extrainit ne "" );
    if ( is_connected() ) {
         do_error("Session in progress");
         return 1;
    }
    if ( ! -d "/var/spool/lock" ) {
	make_path("/var/spool/lock", {mode => 0775});
 	chown(66, 68, "/var/spool/lock");	
    }
    unlink("/var/spool/lock/LCK..cuaU0");
    system("sed -e \'s/PHONE/$phone/g\' < $PPPCONFTMPL | sed -e \'s/IDLETIMEOUT/$idle/g\' | sed -e \'s/EXTRAINIT/$extrainit/g\' | sed -e \'s/SPEED/$speed/g\' | sed -e \'s/AUTHNAME/$authname/g\' | sed -e \'s/AUTHKEY/$authkey/g\' > $PPPCONF");
    my $status = system("$PPP $arg 1> /dev/null 2>&1 ");
    if ($status) {
	 do_error("Dialup failed");
	 return 2;
    } 
    sleep(3);  # allow time for the firewall rules to be modified
    ok();
    return 0;
}

sub do_hangup($)
{
    my ($silent) = @_;

    if ( $childpid ) {
	kill 3, $childpid;
	$childpid = 0;
	ok();
	return 0;
    }

    if ( is_connected() ) {
        my $status = system("$PPPCTL close 1>/dev/null 2>&1");
        ok();
	return 0;
    } else {
	if ( $silent ) {
	    ok();
	} else {
	    do_error("Not connected");
	}
	return 1;
    }
}

sub do_server()
{
    debug("Starting");
    my $loggedin = 0;
    my $line;

    alarm($UTIMEOUT);
    while ( $line = $clientin->getline() ) {
	$ipderror = 0;
        alarm($UTIMEOUT);
	chomp($line);
	$line =~ s/\n//;
	$line =~ s/\r//;
        my @args = split (/=/, $line);
        debug("cmd: \"@args\"");
	
      SWITCH: {
	  my $cmd = lc($args[0]);
          shift (@args);
	  last SWITCH if ( "$cmd" eq "" );  
          if ( "$cmd" =~ /bye/ || "$cmd" =~ /quit/ || "$cmd" =~ /close/ || "$cmd" =~ /done/ ) {
              goto DONE;
          }
          if ( "$cmd" eq "at" && "$args[0]" eq "" ) {
	      ok();
	      last SWITCH;
          }
	  if ( "$cmd" eq "at+login" || "$cmd" eq "at+passwd" ) {
		if ( $args[0] eq "" ) {
			do_error("Missing argument(s)");
			last SWITCH;
		}
		$loggedin =  do_php("-l \'$args[0]\'");
		last SWITCH;
	  }
          if ( "$cmd" =~ /ath/ ) {
 	     do_hangup(1);
    	     last SWITCH;
 	  }
	  if ( ! $loggedin ) {
		do_error("Please login");
		last SWITCH;
	  }
	  if ( "$cmd" eq "at+reset" ) {
		do_php("-c");
		last SWITCH;
	  }
	  if ( "$cmd" eq "at+sum" ) {
	      do_php("-m");
	      last SWITCH;
	  }
	  if ( "$cmd" eq "at+ver" ) {
	      my @version = `cat $ver`;
	      chomp(@version);
	      do_send("@version");
	      ok();
	      last SWITCH;
	  }
	  if ( "$cmd" eq "at+firewall" ) {
 	      if ( $args[0] eq "" ) {
		  do_error("Missing argument(s)");
		  last SWITCH;
	      }
 	      my @params = split (/,/, $args[0]);
	      if ( @params == 3 ) {
		   ($all, $dns, $ftp, $remote) = @params;
		   do_php("-a $all -f $ftp -d $dns");
	      } else {
		   do_error("Invalid parameters");
	      }
	      last SWITCH;
	  }
	  if ( "$cmd" eq "at+remotemgt" ) {
              if ( $args[0] eq "" ) {
                  do_error("Missing argument(s)");
                  last SWITCH;
              }
   	      do_php("-r $args[0]");
	      last SWITCH;
	  }
	  if ( "$cmd" eq "at+ssid" ) {
 	      if ( $args[0] eq "" ) {
		  do_error("Missing argument(s)");
		  last SWITCH;
	      }
	      do_php("-s $args[0]");
	      last SWITCH;
	  }
	  if ( "$cmd" eq "at+ssidpsk" ) {
 	      if ( $args[0] eq "" ) {
		  do_error("Missing argument(s)");
		  last SWITCH;
	      }
	      do_php("-p $args[0]");
	      last SWITCH;
	  }
	  if ( "$cmd" eq "at+adminpw" ) {
 	      if ( $args[0] eq "" ) {
		  do_error("Missing argument(s)");
		  last SWITCH;
	      }
	      do_php("-b $args[0]");
	      last SWITCH;
	  }
	  if ( "$cmd" eq "at+idle" ) {
 		if ( $args[0] eq "" ) {
		      do_error("Missing argument(s)");
		      last SWITCH;
	        }
	 	$idle = "$args[0]";
		last SWITCH;
	  }
	  if ( "$cmd" eq "at+extra" ) {
 		if ( $args[0] eq "" ) {
		      do_error("Missing argument(s)");
		      last SWITCH;
	        }
		$extrainit = "$args[0]";
		last SWITCH;
	  }	
	  if ( "$cmd" eq "at+phone" ) {
 		if ( $args[0] eq "" ) {
		      do_error("Missing argument(s)");
		      last SWITCH;
	        }
		$phone = "$args[0]";
		last SWITCH;
	  }
	  if ( "$cmd" eq "at+speed" || "$cmd" eq "at+baud" ) {
 		if ( $args[0] eq "" ) {
		      do_error("Missing argument(s)");
		      last SWITCH;
	        }
		$speed = "$args[0]";
	  	last SWITCH;
	  }
	  if ( "$cmd" eq "at+auth" ) {
 		if ( $args[0] eq "" ) {
		      do_error("Missing argument(s)");
		      last SWITCH;
	        }
		$authname = "$args[0]";
		last SWITCH;
	  }
	  if ( "$cmd" eq "at+key" ) {
 		if ( $args[0] eq "" ) {
		      do_error("Missing argument(s)");
		      last SWITCH;
	        }
		$authkey = "$args[0]";
		last SWITCH;
	  }
          if ( "$cmd" eq "at+dp" ) {
 	      if ( $args[0] eq "" ) {
	          do_error("Missing argument(s)");
		  last SWITCH;
	      }
	      my @params =  split (/,/, $args[0], -6);
	      if ( @params == 2 ) {
		   ($phone, $speed) = @params;
	      } 
	      if ( @params == 3 ) {
	           ($phone, $speed, $extrainit) = @params;
	      }
	      if ( @params == 4 ) {
	           ($phone, $speed, $authname, $authkey) = @params;
 	      } 
	      if ( @params == 5 ) {
		   ($phone, $speed, $authname, $authkey, $extrainit) = @params;
	      } 
	      if ( @params == 6 ) {
		   ($phone, $speed, $authname, $authkey, $extrainit, $idle) = @params;
	      }
	      if ( @params < 2 || @params > 6 ) {
	      	    do_error("Invalid parameters");
	      } else {
		    ok();
	      }
	      last SWITCH;
	  } 
	  if ( "$cmd" =~ /atd/ && $args[0] eq "" ) {
	        do_error("Missing argument(s)");
		last SWITCH;
	  }
          $arg = lc($args[0]);
	  if ( "$cmd" =~ /atd/ && ("$arg" eq "openport" || "$arg" eq "bgan" || $arg eq "fbb") ) {
	      do_openport();
	      last SWITCH;
	  }
	  if ( "$cmd" =~ /atd/ && "$arg" eq "iridium" ) {
	      $phone     = '008816000025' if ( "$phone" eq "" );
	      $authname  = 'IRIDIUM' if ( "$authname" eq "" );
	      $authkey   = 'IRIDIUM' if ( "$authkey" eq "" );
	      $speed     = '19200' if ( "$speed" eq "" );
	      do_dialup("$arg");
	      last SWITCH;
	  }
          if ( "$cmd" =~ /atd/ && "$arg" eq "globalstar" ) {
              $phone     = '\\\\#777' if ( "$phone" eq "" );
              $speed     = '38400' if ( "$speed" eq "" );
              do_dialup("generic");
              last SWITCH;
          }
          if ( "$cmd" =~ /atd/ && "$arg" eq "isatphone" ) {
              $phone     = '28' if ( "$phone" eq "" );
              $authname  = 'INMARSAT' if ( "$authname" eq "" );
              $authkey   = 'INMARSAT' if ( "$authkey" eq "" );
              $speed     = '2400' if ( "$speed" eq "" );
              do_dialup("isatphone");
              last SWITCH;
          }
	  if ("$cmd" =~ /atd/ && ("$arg" eq "" || "$arg" eq "generic" || "$arg" eq "genericwinit" ) ) {
	      do_error("undefined phone number") if ( "$phone" eq "" );
	      do_error("undefined speed") if ( "$speed" eq "" );
	      do_error("undefined extra init") if ( "$arg" eq "genericwinit" && "$extrainit" eq "" );
	      do_dialup("$arg") if ( !$ipderror );
	      last SWITCH;
	  }
	  if ( "$cmd" =~ /status/ ) {
	     do_status();
	     ok();
	     last SWITCH;
	  }
          do_error("Unknown command \"$cmd\"");
      }
    }

DONE:
    debug("Done.");
}

#
# options
# b - baud
# n - phone number
# u - PAP/CHAP username
# p - PAP/CHAP password
# i - extra initialization string
# t - idle timeout
# flags
# S - server mode used when executing from inetd
# d - dial out 
# s - status 
# h - hangup
getopts("Sdhsb:n:u:p:i:t:");

sub usage()
{
    do_error("Usage: $ARGV[0] -[d|h|s|S] [-b baud] [-n phonenum] [-u username] [-p password] [-i extrainit]");
    exit(1);
}

if ( !($opt_d || $opt_s || $opt_h || $opt_S) || ($opt_d && $opt_s && $opt_h && $opt_S) ) {
    usage();
}

$phone     = $opt_n if ( $opt_n );
$authname  = $opt_u if ( $opt_u );
$authkey   = $opt_p if ( $opt_p );
$extrainit = $opt_i if ( $opt_i );
$speed     = $opt_b if ( $opt_b );
$idle	   = $opt_t if ( $opt_t );

$phone =~ s/\#/\\\\\#/;

do_server() if ( $opt_S );
exit(do_hangup(0)) if ( $opt_h );
exit(do_dialup("generic")) if ( $opt_d);
exit(do_status()) if ( $opt_s );


