#!/usr/local/bin/php -q
<?php
/* $Id$ */
/*
ipdial interface for managing wxa config

*/

error_reporting(E_STRICT);

/* parse the configuration and include all functions used below */
require_once("globals.inc");
require_once("config.inc");
require_once("functions.inc");

$d_filterconfdirty_path = "{$g['tmp_path']}/filter_dirty";

$options = array();
$newssidpw = "";
$ssid  = "";
$firewall_ftp   = "";
$firewall_all   = "";
$firewall_dns   = "";
$firewall_wan	= "";
$password = "";
$mdsum = 0;
$factory_defaults = 0;
$newadminpw = "";

function usage()
{
	echo "Usage: $argv[0] [-c] [-m] [-b new_adminpw] [-l adminpw] [-p new_ssid_pw] [-s new_ssid] [-w enable|disable] [-f enable|disable] [-a enable|disable] [-d enable|disable]\n";
	echo "-p: new ssid password\n";
	echo "-s: set new ssid\n";
	echo "-r: enable/disable remote management via WAN\n";
	echo "-f: enable/disable ftp firewall rule\n";
	echo "-a: enable/disable firewall\n";
	echo "-d: enable/disable DNS\n";
	echo "-m: computer md5 sum of filter array\n";
	echo "-l: login user. webGui password return ACK if password matches. Otherwise ERROR:\n";
	echo "-c: set factory defaults\n";
	echo "-b: set new admin password\n";

	exit();
}

function parse_args() 
{
	global $factory_defaults, $newadminpw, $loginpw, $mdsum, $newssidpw, $ssid, $firewall_all, $firewall_ftp, $firewall_dns, $firewall_wan;

	$options = getopt("cmp:s:r:f:a:d:l:b:");

#	var_dump($options);

	if ( sizeof($options) == 0 )
		usage();

	if ( $options["p"] != "" ) {
		$newssidpw = $options["p"];
	}

	if ( $options["s"] != "" ) {
		$ssid = $options["s"];
	}

	if ( $options["r"] != "" ) {
		$firewall_wan = $options["r"];
	}

	if ( $options["a"] != "" ) {
		$firewall_all = $firewall_dns = $firewall_ftp = $options["a"];
	}

	if ( $options["f"] != "" ) {
		$firewall_ftp = $options["f"];
	}

	if ( $options["d"] != "" ) {
		$firewall_dns = $options["d"];
	}

	if ( $options["l"] != "" ) {
		$loginpw = $options["l"];
	}

	if ( isset($options["m"]) ) {
  	    $mdsum = 1;
	}

	if ( isset($options["c"]) ) {
	   $factory_defaults = 1;
	}

	if ( $options["b"] != "" ) {
		$newadminpw = $options["b"];
	}
}
	
parse_args();

if ( $factory_defaults ) {
	reset_factory_defaults();
	system_reboot();
}
	
$wlancfg = &$config['interfaces']['opt1']['wireless'];
$cfgssid = &$wlancfg['ssid'];
$cfgpw   = &$wlancfg['wpa']['passphrase'];

if ( $ssid != "" || $newssidpw != "" ) {
	if ( $cfgssid == "" ) {
		echo "ERROR: not a wifi router\n";
		exit();
	}
}

if ( $ssid != "" ) {
	$cfgssid = $ssid;
}

if ( $newssidpw  != "" ) {
	$cfgpw = $newssidpw;
}

$filters = &$config['filter'];
$rules = &$config['filter']['rule'];
$rules_new = array();

$mdsumstr = "";

foreach ($rules as $rule) {
	$descr = &$rule['descr'];
	$type  = &$rule['type'];
	$iface = &$rule['interface'];
	if ( $iface == "wan" && $firewall_wan != "" ) 
		$type = $firewall_wan == "enable" ? "pass" : "block";
	if ( $descr == "FTP - DO NOT MODIFY" && $firewall_ftp != "" ) 
		$type = $firewall_ftp == "enable" ? "pass" : "block";
	if ( $descr == "DNS - DO NOT MODIFY" && $firewall_dns != "" )
		$type = $firewall_dns == "enable" ? "pass" : "block";
	if ( $descr == "ALL - DO NOT MODIFY" && $firewall_all != "" )
		$type = $firewall_all == "enable" ? "pass" : "block";
	$rules_new[] = $rule;
	$mdsumstr .= "$type $descr $iface";
}

$rules = $rules_new;

if ( $newssidpw != "" || $ssid != "" || $firewall_all != "" || $firewall_dns != "" || $firewall_ftp != "" || $firewall_wan != "" ) {
	write_config("$argv[0]: Updating config parameters");
	touch($d_filterconfdirty_path);
}

if ( $mdsum ) {
	$sum = md5($mdsumstr);
	echo "SUM$sum\n";
}

if ( $loginpw ) {
	# code taken from /etc/auth.inc
	$authfile = file("{$g['varrun_path']}/htpasswd");

        if(!($line = array_shift(preg_grep("/^.*:.*$/", $authfile)))) {
		echo "ERROR: can't find system password file\n";
		exit();
	}

        /* Get crypted password */
        $matches = "";
        preg_match("/^.*:((\\$1\\$[.\d\w_\/]{8}\\$)[.\d\w_\/]{22})$/", $line, $matches);
        $pass = $matches[1];
        $salt = $matches[2];

        /* Encrypt entered password with salt */
        $authpass = crypt($loginpw, $salt);

         /* And finally validate password */
         if($authpass != $pass) {
		echo "ERROR: Invalid login\n";
		exit();
	 }
}

if ($newadminpw != "") {
	$config['system']['password'] = crypt("$newadminpw");
	sync_webgui_passwords();
}


echo "ACK\n";

?>
